"""

Fixes OVF file that contains extra disk from Vagrant ubuntu/bionic64 image.
Virtual Machines run fine without that disk.

"""

import sys
import os
from lxml import etree

VM_NAME = 'slashdb'

SCRIPTDIR = os.path.dirname(os.path.realpath(__file__))
REPODIR = os.path.abspath(SCRIPTDIR + '/..')


def remove_nodes(nodes):
    for node in nodes:
        node.getparent().remove(node)


def main(ovf_in, ovf_out):
    tree = etree.parse(ovf_in)
    root = tree.getroot()
    ns = {
        'ovf': root.nsmap['ovf'],
        'rasd': root.nsmap['rasd'],
        'vbox': root.nsmap['vbox'],
        'vssd': root.nsmap['vssd']
    }

    nodes_to_remove = []

    try:
        file2 = root.xpath('//ovf:File[@ovf:id="file2"]', namespaces=ns)[0]
    except IndexError:
        pass
    else:
        nodes_to_remove.append(file2)

    try:
        vmdisk2 = root.xpath('//ovf:Disk[@ovf:diskId="vmdisk2"]', namespaces=ns)[0]
        vmdisk2_uuid = vmdisk2.attrib['{%s}uuid' % ns['vbox']]
    except IndexError:
        vmdisk2_uuid = None
        pass
    else:
        nodes_to_remove.append(vmdisk2)

    try:
        item = root.xpath('//ovf:Item/rasd:HostResource[.="/disk/vmdisk2"]', namespaces=ns)[0].getparent()
    except IndexError:
        pass
    else:
        nodes_to_remove.append(item)

    if vmdisk2_uuid:
        try:
            image = root.xpath('//ovf:Image[@uuid="{%s}"]' % vmdisk2_uuid, namespaces=ns)[0].getparent()
        except IndexError:
            pass
        else:
            nodes_to_remove.append(image)

    remove_nodes(nodes_to_remove)

    # rename virtualbox
    root.xpath('//ovf:VirtualSystem', namespaces=ns)[0].attrib['{%s}id' % ns['ovf']] = VM_NAME
    root.xpath('//vssd:VirtualSystemIdentifier', namespaces=ns)[0].text = VM_NAME
    root.xpath('//vbox:Machine', namespaces=ns)[0].attrib['name'] = VM_NAME

    # add NAT
    nat = root.xpath('//ovf:Network//ovf:NAT', namespaces=ns)[0]
    if not root.xpath('//ovf:Network//ovf:NAT/ovf:Forwarding', namespaces=ns):
        nat.append(etree.Element("Forwarding", name="slashdb", proto="1", hostip="127.0.0.1", hostport="8888",
                                 guestip="0.0.0.0", guestport="80"))
        nat.append(
            etree.Element("Forwarding", name="ssh", proto="1", hostip="127.0.0.1", hostport="2222", guestip="0.0.0.0",
                          guestport="22"))

    tree.write(ovf_out, pretty_print=True, xml_declaration=True)


if __name__ == '__main__':
    try:
        ovf_in = sys.argv[1]
    except IndexError:
        ovf_in = None
        print("Missing input OVF file.")
        exit(1)

    try:
        ovf_out = sys.argv[2]
    except IndexError:
        print("Missing path for output OVF file. Will overwrite input.")
        ovf_out = ovf_in

    if ovf_in:
        main(ovf_in, ovf_out)
        exit(0)
