'use strict';

angular.module('sdbgui')
    .component('databases', {
        templateUrl: '/static/components/sdbgui-databases/sdbgui-databases.template.html',
        controller: [
            '$q', '$scope', '$location', 'sortingService', 'slashDB',
            function QueryController($q, $scope, $location, sortingService, slashdb) {
                var ctrl = this;
                ctrl.sort = sortingService;

                this.constructor = function () {
                    ctrl.dbDict = {};
                    ctrl.dbList = [];
                    ctrl.updateList();
                };

                this.updateList = function () {
                    slashdb.get('/db.json', {
                            params: {
                                'guidata': ''
                            }
                        }, false, false)
                        .then(function (response) {
                            var dbList = [],
                                dbDict = {},
                                dataKeys = Object.keys(response.data),
                                refid_prefix = slashdb.settings.refid_prefix,
                                db, dataKey;

                            for (var idx = 0, dataKeysL = dataKeys.length; idx < dataKeysL; idx++) {
                                dataKey = dataKeys[idx];
                                if (dataKey !== refid_prefix) {
                                    db = angular.extend({
                                        id: dataKey
                                    }, response.data[dataKey]);
                                    dbList.push(db);
                                    dbDict[dataKey] = db;
                                }
                            }

                            ctrl.listAlert = '';
                            // update ctrl stuff only once
                            ctrl.dbList = dbList;
                            ctrl.dbDict = dbDict;

                            // refresh slashDB.settings
                            slashdb.setupSettings();
                        })
                        .catch(function (response) {
                            ctrl.listAlert = response.headers('Warning');
                            ctrl.dbList = [];
                        });
                };

                this.toggleDBState = function (dbConfig) {
                    var connectStatus = dbConfig.connect_status;
                    dbConfig.connect_status = '';
                    slashdb.toggleDB(dbConfig.id, connectStatus)
                        .then(function () {
                            ctrl.updateList();
                        })
                        .catch(function (response) {
                            // only if not Request or Gateway Timeout
                            if ([408, 504].indexOf(response.status) < 0) {
                                dbConfig.connect_status = ctrl.Statuses.Failed;
                                ctrl.listAlert = response.headers('Warning');
                            }
                        });
                };

                this.Statuses = {
                    Failed: 'Failed',
                    Connected: 'Connected',
                    Disconnected: 'Disconnected',
                    all: []
                };
                this.Statuses.all = [this.Statuses.Failed, this.Statuses.Connected, this.Statuses.Disconnected];

                this.getStatusStyle = function (status) {
                    if (status == ctrl.Statuses.Connected) return 'bg-success';
                    if (status == ctrl.Statuses.Failed) return 'bg-danger';
                };

                this.isLoading = function (dbConfig) {
                    return ctrl.Statuses.all.indexOf(dbConfig.connect_status) < 0;
                };

                this.constructor();
            }
        ]
    });