// @ts-check
'use strict';

// @ts-ignore
angular.module('sdbgui')
    .component('dbconfigs', {
        templateUrl: '/static/components/sdbgui-dbconfigs/sdbgui-dbconfigs.template.html',
        controller: ['$uibModal', 'sortingService', 'slashDB', function DbConfigController($uibModal, sortingService, slashdb) {
            var ctrl = this;
            ctrl.sort = sortingService;

            ctrl.addConfigVisible = false;
            ctrl.init = function () {
                ctrl.updateConfigList();
            };

            ctrl.updateConfigList = function () {
                slashdb.get('/dbdef.json', {
                        params: {
                            guidata: ''
                        }
                    }, false, false)
                    .then(function (response) {
                        var dbconfigs = [];
                        ctrl.addConfigVisible = response.data.create_database;
                        // @ts-ignore
                        angular.forEach(response.data.dbdefs, function (dbconf) {
                            dbconfigs.push(dbconf);
                        });
                        ctrl.listAlert = '';
                        ctrl.dbconfigs = dbconfigs;
                    })
                    .catch(function (response) {
                        ctrl.listAlert = response.headers('Warning');
                        ctrl.dbconfigs = [];
                    });
            };

            ctrl.Statuses = {
                Failed: 'Failed',
                Connected: 'Connected',
                Disconnected: 'Disconnected',
                all: []
            };
            ctrl.Statuses.all = [ctrl.Statuses.Failed, ctrl.Statuses.Connected, ctrl.Statuses.Disconnected];

            ctrl.getStatusStyle = function (status) {
                if (status == ctrl.Statuses.Connected) return 'bg-success';
                if (status == ctrl.Statuses.Failed) return 'bg-danger';
            };

            ctrl.isLoading = function (dbConfig) {
                return ctrl.Statuses.all.indexOf(dbConfig.connect_status) < 0;
            };

            ctrl.toggleDBState = function (dbConfig) {
                var connectStatus = dbConfig.connect_status;
                dbConfig.connect_status = '';
                slashdb.toggleDB(dbConfig.db_id, connectStatus)
                    .then(function () {
                        ctrl.updateConfigList();
                    })
                    .catch(function (response) {
                        // only if not Request or Gateway Timeout
                        if ([408, 504].indexOf(response.status) < 0) {
                            dbConfig.connect_status = ctrl.Statuses.Failed;
                            ctrl.listAlert = response.headers('Warning');
                        }
                    });
            };

            ctrl.openModal = function (configId) {
                var modalInstance = $uibModal.open({
                    templateUrl: '/static/components/sdbgui-dbconfigs/sdbgui-dbconfigs-modal.template.html',
                    size: 'lg',
                    controller: 'DBConfigModalCtrl',
                    resolve: {
                        configId: function () {
                            return configId;
                        }
                    }
                });
                modalInstance.result
                    .catch(function () {
                        // refresh config list after closing modal
                        ctrl.updateConfigList();
                    });
            };

            ctrl.init();
        }]
    })
    .controller('DBConfigModalCtrl', ['$window', '$timeout', '$q', '$rootScope', '$scope', '$uibModalInstance', 'slashDB', 'configId', 'selectizeService',
        function ($window, $timeout, $q, $rootScope, $scope, $uibModalInstance, slashdb, configId, selectizeService) {
            $scope.newDbConfig = !Boolean(configId);
            $scope.configId = configId;
            $scope.deleteVisible = !$scope.newDbConfig;
            $scope.username = slashdb.settings.user;
            // stores values only for local use, not to be sent to slashdb
            $scope.dbtypes = slashdb.settings.dbtypes;
            var allCharsets = [];
            var mysqlCharsets = [];

            slashdb.settings.py_encodings.forEach(function(charset){
                allCharsets.push(charset.label);
                if (charset.mysql) mysqlCharsets.push(charset.label)
            });

            $scope.required_fields = {};

            $scope.getCharsets = function(dbType){
                if (dbType == 'mysql') {
                    return mysqlCharsets;
                } else {
                    return allCharsets;
                }
            };

            /** @type {{[a: string]: (string | boolean) }} */
            var initExtraConfigData = {
                disabledId: false,
                host: '',
                port: '',
                name: ''
            };
            // @ts-ignore
            $scope.extraConfigData = angular.extend({}, initExtraConfigData);

            var init = function () {
                $scope.selectizeService = selectizeService;
                selectizeService.updateUserOptions(null);
                $scope.okVisible = false;
                $scope.modalAlert = '';
                resetDbConfig();
                getDbConfig();
            };

            /** @returns {Promise} */
            var getDbConfig = function () {
                if ($scope.newDbConfig) {
                    return $q(function (resolve, reject) {
                        resolve();
                    });
                }

                return $q(function (resolve, reject) {
                    slashdb.get('/dbdef/' + $scope.configId + '.json', {
                            params: {
                                guidata: ''
                            }
                        }, false, false)
                        .then(function (response) {
                            $scope.configData = response.data;
                            $scope.extraConfigData.disabledId = true;
                            $scope.deleteVisible = $scope.configData.editable;
                            selectizeService.updateUserOptions($scope.configData);
                            var dbEncoding = $scope.configData.db_encoding;
                            if ($scope.isSQLAlchemy()) updateExtraConfigData();
                            if ($scope.configData.db_type == 'mysql'){
                                if (mysqlCharsets.indexOf(dbEncoding) < 0) mysqlCharsets.push(dbEncoding);
                            } else {
                                if (allCharsets.indexOf(dbEncoding) < 0) allCharsets.push(dbEncoding);
                            }
                            // refresh slashDB.settings
                            slashdb.setupSettings();
                            resolve();
                        });
                });
            };

            var resetDbConfig = function () {
                // stores database definition model, some initial values
                $scope.configData = {
                    db_type: 'sqlite',
                    autoload: false,
                    autoconnect: true,
                    editable: true
                };
                // @ts-ignore
                $scope.extraConfigData = angular.extend({}, initExtraConfigData);
            };

            var updateExtraConfigData = function (override) {
                // Updates $scope.extraConfigData to display gui correctly.
                var parser = $window.document.createElement('a');
                parser.href = 'http://' + $scope.configData.connection; // hack for dummy parser that does not know database protocols
                if (!override) {
                    var port = parser.port ? ':' + parser.port : '';
                    override = $scope.configData.connection !== parser.hostname + port + '/' + parser.pathname.slice(1);
                }
                $scope.extraConfigData = {
                    disabledId: $scope.extraConfigData.disabledId,
                    host: parser.hostname,
                    port: parser.port,
                    name: parser.pathname.slice(1),
                    override: override
                };
            };

            /** @returns {boolean} */
            $scope.requiresCredentials = function () {
                return $scope.configData.use_user_db_credentials_for_mapping &&
                    $scope.configData.db_type !== 'sqlite' &&
                    $scope.configData.autoload &&
                    $scope.isSQLAlchemy();
            };

            $scope.isSQLAlchemy = function () {
                return ['mssql', 'oracle', 'mysql', 'sqlite', 'postgresql', 'db2'].indexOf($scope.configData.db_type) >= 0;
            };

            $scope.updateConnectionString = function () {
                if ($scope.extraConfigData.override) {
                    updateExtraConfigData($scope.extraConfigData.override);
                } else {
                    var port = $scope.extraConfigData.port ? ':' + $scope.extraConfigData.port : '';
                    $scope.configData.connection = $scope.extraConfigData.host + port + '/' + $scope.extraConfigData.name;
                }

            };
            $scope.delete = function () {
                slashdb.deleteDBDef($scope.configId)
                    .then(function (response) {
                        $uibModalInstance.dismiss();

                        // refresh slashDB.settings
                        slashdb.setupSettings();
                    })
                    .catch(function (response) {
                        $scope.modalAlert = response.headers('Warning');
                    });
            };

            /** 
             * @param {string} dbId
             * @param {boolean} useForMapping
             */
            var sendAddBtnMsg = function (dbId, useForMapping) {
                var addBtnMsg = useForMapping ? '' :
                    'Click to associate this user with database "' + dbId + '" login / password.';
                $rootScope.$broadcast('open-user-settings-modal', addBtnMsg);
            };

            function arrayToObject(array){
                    var key, obj={};
                    for (var i = 0; i < array.length; i++){
                        key = array[i];
                        obj[key] = true;
                    }

                    return obj;
                }

            $scope.save = function () {
                $scope.modalAlert = '';
                $scope.configData.override_switch = $scope.extraConfigData.override;

                if ($scope.newDbConfig) {
                    slashdb.createDBDef($scope.configData)
                        .then(function (response) {
                            /** @type {boolean} */
                            var useForMapping = $scope.configData.use_user_db_credentials_for_mapping || false;

                            blinkOK();
                            $scope.newDbConfig = false;
                            $scope.configId = $scope.configData.db_id;
                            getDbConfig().then(function () {
                                sendAddBtnMsg($scope.configId, useForMapping);
                            });
                        })
                        .catch(function (response) {
                            $scope.modalAlert = response.headers('Warning');
                            $scope.required_fields = arrayToObject(response.data.required_fields);
                        });
                } else {
                    slashdb.updateDBDef($scope.configData.db_id, $scope.configData)
                        .then(function (response) {
                            blinkOK();
                            getDbConfig();
                        })
                        .catch(function (response) {
                            $scope.modalAlert = response.headers('Warning');
                            $scope.required_fields = arrayToObject(response.data.required_fields);
                        });
                }
            };

            $scope.close = function () {
                $uibModalInstance.dismiss();
            };

            var blinkOK = function () {
                $scope.okVisible = true;
                $timeout(function () {
                    $scope.okVisible = false;
                }, 1000);
            };
            init();
        }
    ]);